/* ******************************************************************************

	FILE:			SSBaseApp.h
	AUTHOR:			Jean-Sbastien Perrier

	DESCRIPTION:	Base class for an OpenGL screen saver application. This
					class handles everything a screen saver should do. It creates
					the window and handles the OpenGL context creation. A lot of 
					things have been hard coded in order to simplify the class 
					(such as the pixel format and frame buffer configuration for
					the Opengl context).

	$History::                                                                  $

****************************************************************************** */
#ifdef _MSC_VER
	#pragma once
#endif
#ifndef SSBASEAPP_H
#define SSBASEAPP_H

// Includes ****************************************************************** //

// Defines ******************************************************************* //

// Forward definitions ******************************************************* //

// Types ********************************************************************* //

////////////////////////////////////////////////////////////////////////////////
// CLASS:			SSBaseApp
// INHERITS FROM:	None
// DESCRIPTION:		Base class for an OpenGL screen saver application. This
//					class contains pure virtual functions, it has to be
//					derived in order to be instanciated.
// REMARKS:			None.
////////////////////////////////////////////////////////////////////////////////
class SSBaseApp
{
public:
	// Running mode enumeration.
	enum ModeEnum 
	{ 
		smNone, 
		smConfig, 
		smPassword, 
		smPreview, 
		smSaver
	};

private:
	char m_szWindowName[256];	// Application window's name
	char m_szWindowClassName[256];	// Window class name.
	ModeEnum m_eMode;	// Application mode (config, preview, saver, etc).

	HINSTANCE m_hInstance;	// Application's instance handle.
	HWND m_hWnd;			// Window handle of the application

	static SSBaseApp *s_pInstance;	// Required for windows call back function.
	
	// Stuff needed for screen saver management.
	DWORD m_dwPasswordDelay;	// in seconds
	DWORD m_dwMouseThreshold;	// in pixels
	BOOL  m_bMuteSound;

	POINT m_InitCursorPos;		//	Initial cursor position
	DWORD m_dwInitTime;			// in ms	
	BOOL  m_bIsDialogActive;
	BOOL  m_bReallyClose;		// for NT, so we know if a WM_CLOSE came from us or it.


private:
	// The window's callback function has to be static so that we 
	// can give it's resolved address to the window class.
	static LRESULT CALLBACK SaverWindowProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
	
public:
	SSBaseApp(HINSTANCE a_hInstance);
	virtual ~SSBaseApp() {}

	HWND ProcessCommandLine(char *a_szCmd);

	void OnCreate(HWND a_hWnd);
	void RunScreenSaver(HWND a_hParentWnd, int nCmdShow);
	void CloseSaverWindow();
	void StartDialog();
	void EndDialog();

	// Get methods
	const char *GetWindowName() const {return m_szWindowName;}
	const char *GetWindowClassName() const {return m_szWindowClassName;}
	ModeEnum GetMode() const {return m_eMode;}
	HWND GetWindowHandle() const {return m_hWnd;}

	// Set methods
	void SetWindowHandle(HWND a_hWnd) {m_hWnd = a_hWnd;}

	// Global methods
	static SSBaseApp *GetGlobalInstance() { return s_pInstance;}
	static void SetGlobalInstance(SSBaseApp *a_pInstance) {s_pInstance = a_pInstance;}

	// Derived functions
	virtual void ReadConfigRegistry() = 0;
	virtual void WriteConfigRegistry() = 0;
	virtual bool InitGL(int w, int h) = 0;
	virtual void ReleaseGL() = 0;
	virtual void Update() = 0;
	virtual void Display() = 0;
};

// Consts ******************************************************************** //

// Globals ******************************************************************* //

// Function Prototypes ******************************************************* //

// Inline Functions ********************************************************** //

#endif // SSBASEAPP_H
